library angular2.src.core.compiler.shadow_dom_emulation.content_tag;

import '../../annotations/annotations.dart' show Decorator;
import './light_dom.dart' as ldModule;
import 'package:angular2/di.dart' show Inject;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import 'package:angular2/src/facade/lang.dart' show isPresent;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper;
import 'package:angular2/src/core/dom/element.dart' show NgElement;

class ContentStrategy {
  List nodes;
  insert(List nodes) {}
}
class RenderedContent extends ContentStrategy {
  static var _lazyScriptTemplate;
  var beginScript;
  var endScript;
  RenderedContent(contentEl) : super() {
    this._replaceContentElementWithScriptTags(contentEl);
    this.nodes = [];
  }
  _scriptTemplate() {
    if (!isPresent(RenderedContent._lazyScriptTemplate)) {
      RenderedContent._lazyScriptTemplate =
          DOM.createScriptTag('type', 'ng/content');
    }
    return RenderedContent._lazyScriptTemplate;
  }
  insert(List nodes) {
    this.nodes = nodes;
    DOM.insertAllBefore(this.endScript, nodes);
    this._removeNodesUntil(
        ListWrapper.isEmpty(nodes) ? this.endScript : nodes[0]);
  }
  _replaceContentElementWithScriptTags(contentEl) {
    this.beginScript = DOM.clone(this._scriptTemplate());
    this.endScript = DOM.clone(this._scriptTemplate());
    DOM.insertBefore(contentEl, this.beginScript);
    DOM.insertBefore(contentEl, this.endScript);
    DOM.removeChild(DOM.parentElement(contentEl), contentEl);
  }
  _removeNodesUntil(node) {
    var p = DOM.parentElement(this.beginScript);
    for (var next = DOM.nextSibling(this.beginScript);
        !identical(next, node);
        next = DOM.nextSibling(this.beginScript)) {
      DOM.removeChild(p, next);
    }
  }
}
class IntermediateContent extends ContentStrategy {
  ldModule.LightDom destinationLightDom;
  IntermediateContent(ldModule.LightDom destinationLightDom) : super() {
    this.destinationLightDom = destinationLightDom;
    this.nodes = [];
  }
  insert(List nodes) {
    this.nodes = nodes;
    this.destinationLightDom.redistribute();
  }
}
@Decorator(selector: 'content')
class Content {
  String select;
  ContentStrategy _strategy;
  Content(@Inject(ldModule.DestinationLightDom) destinationLightDom,
      NgElement contentEl) {
    this.select = contentEl.getAttribute('select');
    this._strategy = isPresent(destinationLightDom)
        ? new IntermediateContent(destinationLightDom)
        : new RenderedContent(contentEl.domElement);
  }
  List nodes() {
    return this._strategy.nodes;
  }
  insert(List nodes) {
    this._strategy.insert(nodes);
  }
}
