library angular2.src.forms.model;

import 'package:angular2/src/facade/lang.dart' show isPresent;
import 'package:angular2/src/facade/collection.dart' show Map, StringMapWrapper;
import './validators.dart' show nullValidator, controlGroupValidator;

const VALID = "VALID";
const INVALID = "INVALID";
class AbstractControl {
  dynamic _value;
  String _status;
  var _errors;
  bool _dirty;
  ControlGroup _parent;
  Function validator;
  AbstractControl([Function validator = nullValidator]) {
    this.validator = validator;
    this._dirty = true;
  }
  get value {
    this._updateIfNeeded();
    return this._value;
  }
  get status {
    this._updateIfNeeded();
    return this._status;
  }
  get valid {
    this._updateIfNeeded();
    return identical(this._status, VALID);
  }
  get errors {
    this._updateIfNeeded();
    return this._errors;
  }
  setParent(parent) {
    this._parent = parent;
  }
  _updateIfNeeded() {}
  _updateParent() {
    if (isPresent(this._parent)) {
      this._parent._controlChanged();
    }
  }
}
class Control extends AbstractControl {
  Control(dynamic value, [Function validator = nullValidator])
      : super(validator) {
    this._value = value;
  }
  updateValue(dynamic value) {
    this._value = value;
    this._dirty = true;
    this._updateParent();
  }
  _updateIfNeeded() {
    if (this._dirty) {
      this._dirty = false;
      this._errors = this.validator(this);
      this._status = isPresent(this._errors) ? INVALID : VALID;
    }
  }
}
class ControlGroup extends AbstractControl {
  var controls;
  var optionals;
  ControlGroup(controls,
      [optionals = null, Function validator = controlGroupValidator])
      : super(validator) {
    this.controls = controls;
    this.optionals = isPresent(optionals) ? optionals : {};
    this._setParentForControls();
  }
  include(String controlName) {
    this._dirty = true;
    StringMapWrapper.set(this.optionals, controlName, true);
  }
  exclude(String controlName) {
    this._dirty = true;
    StringMapWrapper.set(this.optionals, controlName, false);
  }
  contains(String controlName) {
    var c = StringMapWrapper.contains(this.controls, controlName);
    return c && this._included(controlName);
  }
  _setParentForControls() {
    StringMapWrapper.forEach(this.controls, (control, name) {
      control.setParent(this);
    });
  }
  _updateIfNeeded() {
    if (this._dirty) {
      this._dirty = false;
      this._value = this._reduceValue();
      this._errors = this.validator(this);
      this._status = isPresent(this._errors) ? INVALID : VALID;
    }
  }
  _reduceValue() {
    var newValue = {};
    StringMapWrapper.forEach(this.controls, (control, name) {
      if (this._included(name)) {
        newValue[name] = control.value;
      }
    });
    return newValue;
  }
  _controlChanged() {
    this._dirty = true;
    this._updateParent();
  }
  bool _included(String controlName) {
    var isOptional = StringMapWrapper.contains(this.optionals, controlName);
    return !isOptional || StringMapWrapper.get(this.optionals, controlName);
  }
}
